#include "svg_personal.h"
#include <vector>
#include <iostream>

using namespace std;

void
svg_begin(double width, double height) {
    cout << "<?xml version='1.0' encoding='UTF-8'?>\n";
    cout << "<svg ";
    cout << "width='" << width << "' ";
    cout << "height='" << height << "' ";
    cout << "viewBox='0 0 " << width << " " << height << "' ";
    cout << "xmlns='http://www.w3.org/2000/svg'>\n";
}

void
svg_end() {
    cout << "</svg>\n";
}

void
svg_text(double left, double baseline, string style, string text) {
    cout << "<text x='" << left << "' y='" << baseline << "' text-decoration='" << style << "'>" << text << "</text>";
}

void
svg_rect(double x, double y, double width, double height,
              string stroke = "black", string fill = "orange"){
    cout << "<rect x='" << x << "' y='" << y << "' width='" << width << "' height='" << height << "' stroke='" << stroke << "' fill='" << fill << "'/>";
}

string
styled_text_func (){
    string text_style;
    cerr << "Do you want to use any text style? (Yes/No)" << endl;
    cin >> text_style;
    if (text_style == "No") {
        text_style = "none";
    }
    else {
        cerr << "Which text style do you want to use?" << endl;
        cin >> text_style;
        while ((text_style != "none") && (text_style != "underline") && (text_style != "overline") && (text_style != "line-through")){
            cerr << "You entered the wrong style name. Try again." << endl;
            cin >> text_style;
        }
    }
    return text_style;
}

void
show_histogram_svg(const vector<size_t>& bins, std::string text_style) {
    const auto IMAGE_WIDTH = 400;
    const auto IMAGE_HEIGHT = 300;
    const auto TEXT_LEFT = 20;
    const auto TEXT_BASELINE = 20;
    const auto TEXT_WIDTH = 50;
    const auto BIN_HEIGHT = 30;
    const auto BLOCK_WIDTH = 10;
    svg_begin(IMAGE_WIDTH, IMAGE_HEIGHT);
    double top = 0;
    double maxbin = bins[0];
    for (size_t bin : bins) {
        if (maxbin < bin){
            maxbin = bin;
        }
    }
    for (size_t bin : bins) {
        double bin_width = BLOCK_WIDTH * bin;
        if ((IMAGE_WIDTH - TEXT_WIDTH) < (bin * BLOCK_WIDTH)){
            bin_width = (IMAGE_WIDTH - TEXT_WIDTH) * ( bin / maxbin );
        }
        svg_text(TEXT_LEFT, top + TEXT_BASELINE, text_style, to_string(bin));
        svg_rect(TEXT_WIDTH, top, bin_width, BIN_HEIGHT);
        top += BIN_HEIGHT;
    }
    svg_end();
}
